/*
*
* Copyright 2012  Samsung Electronics Co., Ltd
*
* Licensed under the Flora License, Version 1.1 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*    http://floralicense.org/license/
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*
*/

#include <stdlib.h>
#include <stdio.h>
#include <appcore-efl.h>
#include <Elementary.h>
#include <dlog.h>
#include <gravel.h>
#include <extended-elm.h>
#include <supplement.h>
#include <memo-assist.h>
#include <memo_string.h>

typedef struct __doodle_editor_t {
    Evas_Object *win;
    Evas_Object *parent;
    Memo_Component_Callback cb;
    void *data;
    memo_data_t *record;
    char buf[MEMO_BUFFER_SIZE]; /* for temporarily usage only */
    Eina_Bool savable;
    Eina_Bool erase;    /* erase mode */
    Eina_Bool comment;    /* edit comment mode */
    Eina_Bool doodle_init;
    Eina_Bool stroke_down;    /* see comment in _on_doodle_move_cb */
    int stroke_size;
    int stroke_color;

    /* content */
    Evas_Object *body_main;
    Evas_Object *doodle;
    Evas_Object *toolbar;
    Evas_Object *btn_size;
    void *h_ss;
    Evas_Object *btn_color;
    void *h_cs;
    Evas_Object *edit_field;

    /* entry */
    Evas_Object *entry;
    Ecore_IMF_Context *imf_context;
} doodle_editor_t;

static void _stroke_size_selector_del(doodle_editor_t *de);
static void _stroke_color_selector_del(doodle_editor_t *de);

static void _save_record(doodle_editor_t *de)
{
    memo_begin_trans();
    if (de->record->id != -1) {
        memo_mod_data(de->record);
    } else {
        de->record->content = memo_get_doodle_title();
        memo_add_data(de->record);
    }
    /* save doodle to file */
    snprintf(de->buf, MEMO_BUFFER_SIZE, DOODLEDIR "/%d.png", (int)de->record->id);
    evas_object_image_save(de->doodle, de->buf, NULL, NULL);
    memo_end_trans();
}

static void _on_save_yes(void *data, Evas_Object *obj, void *event_info)
{
    doodle_editor_t *de = (doodle_editor_t *)data;
    _save_record(de);
    de->cb(de->data, "save", NULL);
}

static void _on_save_no(void *data, Evas_Object *obj, void *event_info)
{
    doodle_editor_t *de = (doodle_editor_t *)data;
    de->cb(de->data, "cancel", NULL);
}

static Eina_Bool _launch_yes_no_popup(void *data)
{
    doodle_editor_t *de = (doodle_editor_t *)data;
    memo_create_yes_no_popup(de->win, MEMO_I18N_SAVE_MEMO, _on_save_yes, _on_save_no, de);
    return EINA_FALSE;
}

static Eina_Bool _comment_savable(doodle_editor_t *de)
{
    const char *text = elm_entry_entry_get(de->entry);
    Eina_Bool savable = EINA_TRUE;

    if (strcmp(text, "") == 0) {
        savable = EINA_FALSE;
    } else if (de->record->comment != NULL) { /* check if same as init string */
        if (strcmp(de->record->comment, text)==0) {
            savable = EINA_FALSE;
        }
    }
    return savable;
}

static void _stroke_size_selector_callback(void *data, const char *msg, void *event)
{
    doodle_editor_t *de = (doodle_editor_t *)data;
    if (strcmp(msg, "layout") == 0) {
        elm_object_part_content_set(de->body_main, "elm.swallow.tool", (Evas_Object *)event);
    } else if (strcmp(msg, "change") == 0) {
        de->stroke_size = (int)event;
    } else if (strcmp(msg, "flick,down") == 0) {
        _stroke_size_selector_del(de);
        edje_object_signal_emit(elm_layout_edje_get(de->body_main), "expand", "elm");
    }
}

static void _stroke_size_selector_del(doodle_editor_t *de)
{
    if (de->h_ss != NULL) {
        elm_object_part_content_unset(de->body_main, "elm.swallow.tool");
        memo_del_pencil_size_selector(de->h_ss);
        de->h_ss = NULL;
        memo_tool_btn_focus_set(de->btn_size, EINA_FALSE);
    }
}

static void _on_stroke_size_btn_up(void *data, Evas *e, Evas_Object *evas_obj, void *event_info)
{
	doodle_editor_t *de = (doodle_editor_t *)data;
	if (de->h_ss != NULL) { /* stroke size selector already open */
		return;
	}
	if (de->h_cs != NULL) { /* destroy stroke color selector */
		_stroke_color_selector_del(de);
	}
	if (de->erase) { /* deselect eraser */
		de->erase = EINA_FALSE;
		Evas_Object *eo = (Evas_Object *)elm_object_part_content_get(de->toolbar, "elm.swallow.btn3");
		memo_tool_btn_focus_set(eo, de->erase);
	}
	/* load stroke size selector */
	service_h service = NULL;
	service_create(&service);
	snprintf(de->buf, MEMO_BUFFER_SIZE, "%d", de->stroke_size);
	service_add_extra_data(service, "current", de->buf);
	snprintf(de->buf, MEMO_BUFFER_SIZE, "%d", 0xFFFFFFFF);
	service_add_extra_data(service, "bg_color", de->buf);
	de->h_ss = memo_load_pencil_size_selector(de->body_main, service, _stroke_size_selector_callback, de);
	service_destroy(service);
	memo_tool_btn_focus_set(de->btn_size, EINA_TRUE);
	edje_object_signal_emit(elm_layout_edje_get(de->body_main), "abbrev", "elm");
}

void _stroke_color_selector_callback(void *data, const char *msg, void *event)
{
    doodle_editor_t *de = (doodle_editor_t *)data;
    if (strcmp(msg, "layout") == 0) {
        elm_object_part_content_set(de->body_main, "elm.swallow.tool", (Evas_Object *)event);
    } else if (strcmp(msg, "change") == 0) {
        de->stroke_color = (int)event;
    } else if (strcmp(msg, "flick,down") == 0) {
        _stroke_color_selector_del(de);
        edje_object_signal_emit(elm_layout_edje_get(de->body_main), "expand", "elm");
    }
}

static void _stroke_color_selector_del(doodle_editor_t *de)
{
    if (de->h_cs != NULL) {
        elm_object_part_content_unset(de->body_main, "elm.swallow.tool");
        memo_del_color_selector(de->h_cs);
        memo_tool_btn_focus_set(de->btn_color, EINA_FALSE);
        de->h_cs = NULL;
    }
}

static void _on_stroke_color_btn_up(void *data, Evas *e, Evas_Object *evas_obj, void *event_info)
{
	doodle_editor_t *de = (doodle_editor_t *)data;
	if (de->h_cs!= NULL) { /* stroke color selector already open */
		return;
	}
	if (de->h_ss!= NULL) { /* destroy stroke  size selector */
		_stroke_size_selector_del(de);
	}
	if (de->erase) { /* deselect eraser */
		de->erase = EINA_FALSE;
		Evas_Object *eo = (Evas_Object *)elm_object_part_content_get(de->toolbar, "elm.swallow.btn3");
		memo_tool_btn_focus_set(eo, de->erase);
	}
	/* load stroke color selector */
	service_h service = NULL;
	service_create(&service);
	snprintf(de->buf, MEMO_BUFFER_SIZE, "%d", de->stroke_color);
	service_add_extra_data(service, "color", de->buf);
	snprintf(de->buf, MEMO_BUFFER_SIZE, "%d", 0xFFFFFFFF);
	service_add_extra_data(service, "bg_color", de->buf);
	de->h_cs = memo_load_color_selector(de->body_main, service, _stroke_color_selector_callback, de);
	service_destroy(service);
	memo_tool_btn_focus_set(de->btn_color, EINA_TRUE);
	edje_object_signal_emit(elm_layout_edje_get(de->body_main), "abbrev", "elm");
}

static void _input_panel_event_callback(void *data, Ecore_IMF_Context *ctx, int value)
{
    doodle_editor_t *de = (doodle_editor_t *)data;
    if (value == ECORE_IMF_INPUT_PANEL_STATE_HIDE) {
        if (de->comment) {
            edje_object_signal_emit(elm_layout_edje_get(de->body_main), "expand", "comment");
        }
    } else if (value == ECORE_IMF_INPUT_PANEL_STATE_SHOW) {
        edje_object_signal_emit(elm_layout_edje_get(de->body_main), "abbrev", "comment");
        if (!de->comment) { /* first shift to comment mode */
            if (de->record->comment == NULL) { /* remove DUMMY_COMMENT */
                elm_entry_entry_set(de->entry, "");
            }
            de->cb(de->data, "savable", (void *)0);
        }
        de->comment = EINA_TRUE;
        de->cb(de->data, "title", de->record->comment == NULL ? MEMO_I18N_ADD_COMMENT : MEMO_I18N_EDIT_COMMENT);
    }
}

static void _on_entry_content_change(void *data, Evas_Object *obj, void *event_info)
{
    doodle_editor_t *de = (doodle_editor_t *)data;
    if (elm_object_focus_get(de->edit_field)) {
        if (elm_entry_is_empty(obj)) {
            elm_object_signal_emit(de->edit_field, "elm,state,eraser,hide", "elm");
        } else {
            elm_object_signal_emit(de->edit_field, "elm,state,eraser,show", "elm");
        }
    }
    RETIF(!de->comment);
    Eina_Bool savable = _comment_savable(de);
    de->cb(de->data, "savable", (void *)(savable?1:0));
}

// Focused callback will show X marked button and hide guidetext
static void _focused_cb(void *data, Evas_Object *obj, void *event_info)
{
    if (!elm_entry_is_empty(obj))
	elm_object_signal_emit(data, "elm,state,eraser,show", "elm");
    elm_object_signal_emit(data, "elm,state,guidetext,hide", "elm");
}

// Unfocused callback will show guidetext and hide X marked button
static void _unfocused_cb(void *data, Evas_Object *obj, void *event_info)
{
    if (elm_entry_is_empty(obj))
	elm_object_signal_emit(data, "elm,state,guidetext,show", "elm");
    elm_object_signal_emit(data, "elm,state,eraser,hide", "elm");
}

// When X marked button is clicked, empty entry’s contents
static void _eraser_clicked_cb(void *data, Evas_Object *obj, const char *emission, const char *source)
{
    elm_entry_entry_set(data, "");
}

static void _draw_argb32(int *img, int w, int h, int x, int y, int size, int color)
{
    int cx, cy, cw, ch;
    int i, j;

    cx = x;
    cy = y;
    cw = size;
    ch = size;
    CLIP_RECT_TO_RECT(cx, cy, cw, ch, 0, 0, w, h);
    for (i = 0; i < ch; i++) {
        for (j = 0; j < cw; j++) {
            memcpy(img + (i + cy) * w + (cx + j), &color, 4);
        }
    }
}

static void _on_doodle_down_cb(void *data, Evas *e, Evas_Object *obj, void *event_info)
{
    doodle_editor_t *de = (doodle_editor_t *)data;
    if (de->comment) {
        return;
    }
    if (de->h_ss!= NULL) { /* destroy stroke  size selector */
        _stroke_size_selector_del(de);
    }
    if (de->h_cs != NULL) { /* destroy stroke color selector */
        _stroke_color_selector_del(de);
    }
    edje_object_signal_emit(elm_layout_edje_get(de->body_main), "expand", "elm");
    de->stroke_down = EINA_TRUE;
    de->record->has_doodle = 1;
    if (!de->erase) {
        de->savable = EINA_TRUE;
    } else if (de->record->id != -1) { /* edit existing doodle with eraser */
        de->savable = EINA_TRUE;
    }
    de->cb(de->data, "savable", (void *)(de->savable?1:0));
}

static void _on_doodle_move_cb(void *data, Evas *e, Evas_Object *obj, void *event_info)
{
    doodle_editor_t *de = (doodle_editor_t *)data;
    Evas_Event_Mouse_Move *event = (Evas_Event_Mouse_Move *)event_info;
    int *img_data = NULL;
    Evas_Coord x, y, w, h;
    int px = 0;
    int py = 0;
    int i = 0;
    int gap_max = 0;
    int gap_x = 0;
    int gap_y = 0;
    int color = 0;

    /* a little mouse move events will be received before mouse down event,
       if those mouse move event is not filtered,
       a line (from last up point to current move point) will be draw */
    if (!de->stroke_down) {
        return;
    }

    color = de->erase ? 0xFFFFFFFF : de->stroke_color;
    evas_object_geometry_get(de->doodle, &x, &y, &w, &h);
    img_data = evas_object_image_data_get(de->doodle, 1);
    gap_x = event->cur.output.x - event->prev.output.x;
    gap_y = event->cur.output.y - event->prev.output.y;
    gap_max = MAX(abs(gap_x), abs(gap_y));

    for (i = 0; i < gap_max; i++) {
        px = event->prev.output.x + i * gap_x / gap_max;
        py = event->prev.output.y + i * gap_y / gap_max;
        /* mouse position in image */
        px -= x;
        py -= y;
        _draw_argb32(img_data, w, h, px, py, de->stroke_size, color);
    }
    evas_object_image_data_update_add(de->doodle, 0, 0, w, h);
}

static void _on_doodle_up_cb(void *data, Evas *e, Evas_Object *obj, void *event_info)
{
    doodle_editor_t *de = (doodle_editor_t *)data;
    de->stroke_down = EINA_FALSE;
}

/* this function must be execute only once
 * in resize callback we will get the actual size of doodle
 * we need to transfer origin image data to fit the actual canvas size
 */
static void _on_doodle_resize_cb(void *data, Evas_Object *obj, void *event_info)
{
    doodle_editor_t *de = (doodle_editor_t *)data;
    Evas_Coord w, h;
    int *img_data = NULL;

    if (de->doodle_init) {    /* reenter control */
        return;
    } else {
        de->doodle_init = EINA_TRUE;
        evas_object_geometry_get(de->doodle, NULL, NULL, &w, &h);
        evas_object_image_size_set(de->doodle, w, h);
        img_data = evas_object_image_data_get(de->doodle, 1);
        if (img_data != NULL) { /* set background color as white */
            memset(img_data, 0xFFFFFFFF, sizeof(int)*w*h);
            evas_object_image_data_update_add(de->doodle, 0, 0, w, h);
        }
    }
}

static void _on_eraser_up(void *data, Evas *e, Evas_Object *evas_obj, void *event_info)
{
    doodle_editor_t *de = (doodle_editor_t *)data;
    if (de->h_ss!= NULL) { /* destroy stroke  size selector */
        _stroke_size_selector_del(de);
    }
    if (de->h_cs != NULL) { /* destroy stroke color selector */
        _stroke_color_selector_del(de);
    }
    de->erase = !(de->erase);
    Evas_Object *eo = (Evas_Object *)elm_object_part_content_get(de->toolbar, "elm.swallow.btn3");
    memo_tool_btn_focus_set(eo, de->erase);
    edje_object_signal_emit(elm_layout_edje_get(de->body_main), "expand", "elm");
}

static void _on_yes(void *data, Evas_Object *obj, void *event_info)
{
    doodle_editor_t *de = (doodle_editor_t *)data;
    _save_record(de);
    de->cb(de->data, "text", NULL);
}

static void _on_no(void *data, Evas_Object *obj, void *event_info)
{
    doodle_editor_t *de = (doodle_editor_t *)data;
    de->cb(de->data, "text", NULL);
}

static void _on_text_mode(void *data, Evas_Object *obj, void *event_info)
{
    doodle_editor_t *de = (doodle_editor_t *)data;
    if (de->savable) { /* modified, need user confirmation */
        memo_create_yes_no_popup(de->parent, MEMO_I18N_SAVE_MEMO, _on_yes, _on_no, de);
    } else {
        de->cb(de->data, "text", NULL);
    }
}

static void _create_doodle_editor_layout(doodle_editor_t *de, service_h service)
{
    de->body_main = elm_layout_create(de->parent, EDJ_FILE, "doodle_editor");
    de->cb(de->data, "layout", de->body_main);
    /* title */
    de->cb(de->data, "title", de->record->id==-1 ? MEMO_I18N_NEW_DRAWING : MEMO_I18N_EDIT_DRAWING);
    /* date */
    memo_time_format(de->buf, MEMO_BUFFER_SIZE, de->record->modi_time);
    edje_object_part_text_set(elm_layout_edje_get(de->body_main), "elm.text.date", de->buf);

    /* toolbar */
    de->toolbar = elm_swallowed_layout(de->body_main, "elm.swallow.toolbar", EDJ_FILE, "edit_toolbar");
    de->btn_size = elm_swallowed_layout(de->toolbar, "elm.swallow.btn1", EDJ_DIR"/black/memo.edj", "tl_stroke_size");
    evas_object_event_callback_add(de->btn_size, EVAS_CALLBACK_MOUSE_UP, _on_stroke_size_btn_up, de);
    de->btn_color = elm_swallowed_layout(de->toolbar, "elm.swallow.btn2", EDJ_DIR"/black/memo.edj", "tl_stroke_color");
    evas_object_event_callback_add(de->btn_color, EVAS_CALLBACK_MOUSE_UP, _on_stroke_color_btn_up, de);
    Evas_Object *eo = elm_swallowed_layout(de->toolbar, "elm.swallow.btn3", EDJ_DIR"/black/memo.edj", "tl_stroke_eraser");
    evas_object_event_callback_add(eo, EVAS_CALLBACK_MOUSE_UP, _on_eraser_up, de);
    if (!service_key_check(service, "toggle", "disable")) {
        elm_swallowed_button(de->toolbar, "elm.swallow.btn4", MEMO_I18N_TEXT, _on_text_mode, de);
    }

    /* doodle */
    Evas *evas = evas_object_evas_get(de->parent);
    de->doodle = evas_object_image_filled_add(evas);
    evas_object_image_colorspace_set(de->doodle, EVAS_COLORSPACE_ARGB8888);
    evas_object_image_alpha_set(de->doodle, 1);
    /* doodle data */
    if (de->record->has_doodle == 1) {
        snprintf(de->buf, MEMO_BUFFER_SIZE, DOODLEDIR "/%d.png", (int)de->record->id);
        evas_object_image_file_set(de->doodle, de->buf, NULL);
    } else {
        evas_object_render_flush_hook(de->doodle, _on_doodle_resize_cb, de);
    }
    evas_object_event_callback_add(de->doodle, EVAS_CALLBACK_MOUSE_DOWN, _on_doodle_down_cb, de);
    evas_object_event_callback_add(de->doodle, EVAS_CALLBACK_MOUSE_MOVE, _on_doodle_move_cb, de);
    evas_object_event_callback_add(de->doodle, EVAS_CALLBACK_MOUSE_UP, _on_doodle_up_cb, de);
    evas_object_show(de->doodle);
    elm_object_part_content_set(de->body_main, "elm.swallow.doodle", de->doodle);

    /* comment */
    eo = elm_swallowed_layout(de->body_main, "elm.swallow.comment", EDJ_FILE, "doodle_comment");
    //EditField
    Evas_Object *layout = elm_layout_add(eo);
    elm_layout_theme_set(layout, "layout", "editfield", "default");
    elm_object_part_content_set(eo, "elm.swallow.editfield", layout);
    Evas_Object *ef = elm_entry_add(eo);
    elm_object_part_content_set(layout, "elm.swallow.content", ef);
    de->edit_field = layout;

    elm_object_style_set(ef, "font_color_black");
    elm_entry_single_line_set(ef, EINA_TRUE);
    elm_entry_scrollable_set(ef, EINA_TRUE);
    evas_object_size_hint_weight_set(ef, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    evas_object_size_hint_align_set(ef, EVAS_HINT_EXPAND, 0.5);

    de->entry = ef;
    elm_entry_cnp_mode_set(de->entry, ELM_CNP_MODE_PLAINTEXT);
    de->imf_context = elm_entry_imf_context_get(de->entry);
    /* if not hide explicitly, seems hide event of sweep down can not be received */
    ecore_imf_context_input_panel_hide(de->imf_context);
    elm_entry_entry_set(de->entry, de->record->comment == NULL ? MEMO_I18N_ADD_COMMENT : de->record->comment);
    ecore_imf_context_input_panel_event_callback_add(de->imf_context, ECORE_IMF_INPUT_PANEL_STATE_EVENT,
        _input_panel_event_callback, de);
    evas_object_smart_callback_add(de->entry, "changed", _on_entry_content_change, de);
    evas_object_smart_callback_add(de->entry, "preedit,changed", _on_entry_content_change, de);
    evas_object_smart_callback_add(de->entry, "focused", _focused_cb, layout);
    evas_object_smart_callback_add(de->entry, "unfocused", _unfocused_cb, layout);
    elm_object_signal_callback_add(layout, "elm,eraser,clicked", "elm", _eraser_clicked_cb, de->entry);

    /* savable */
    de->cb(de->data, "savable", (void *)(de->savable?1:0));
}

void memo_doodle_editor_time_format_update(void *h_de)
{
    doodle_editor_t *de = (doodle_editor_t *)h_de;
    memo_time_format(de->buf, MEMO_BUFFER_SIZE, de->record->modi_time);
    edje_object_part_text_set(elm_layout_edje_get(de->body_main), "elm.text.date", de->buf);
}

void memo_doodle_editor_cancel(void *h_de)
{
    doodle_editor_t *de = (doodle_editor_t *)h_de;
    if (de->comment) {
        de->comment = EINA_FALSE;
        ecore_imf_context_input_panel_hide(de->imf_context);
        edje_object_signal_emit(elm_layout_edje_get(de->body_main), "expand", "elm");
        if (de->record->comment == NULL) {
            elm_entry_entry_set(de->entry, MEMO_I18N_ADD_COMMENT);
        } else { /* recover origin comment */
            elm_entry_entry_set(de->entry, de->record->comment);
        }
        de->cb(de->data, "title", de->record->id==-1 ? MEMO_I18N_NEW_DRAWING : MEMO_I18N_EDIT_DRAWING);
        de->cb(de->data, "savable", (void *)(de->savable?1:0));
        de->cb(de->data, "comment cancel", NULL);
    } else {
        if (!de->savable) { /* not modified, return directly */
            de->cb(de->data, "cancel", NULL);
        } else {
            _launch_yes_no_popup(de);
        }
    }
}

void memo_doodle_editor_save(void *h_de)
{
    doodle_editor_t *de = (doodle_editor_t *)h_de;
    if (de->comment) {
        de->comment = EINA_FALSE;
        ecore_imf_context_input_panel_hide(de->imf_context);
        edje_object_signal_emit(elm_layout_edje_get(de->body_main), "expand", "elm");
        Eina_Bool savable = _comment_savable(de);
        if (savable) {
            de->record->comment = strdup(elm_entry_entry_get(de->entry));
            if (de->record->id != -1) { /* when edit existing, modification of comment will make record savable */
                de->savable = EINA_TRUE;
            }
        }
        de->cb(de->data, "title", de->record->id==-1 ? MEMO_I18N_NEW_DRAWING : MEMO_I18N_EDIT_DRAWING);
        de->cb(de->data, "savable", (void *)(de->savable?1:0));
        elm_object_focus_set(de->edit_field, EINA_FALSE); /* when disable done button, focus may reset to edit field */
        de->cb(de->data, "comment done", NULL);
    } else {
        _save_record(de);
        de->cb(de->data, "save", NULL);
    }
}

void *memo_load_doodle_editor(Evas_Object *win, Evas_Object *parent, service_h service, Memo_Component_Callback cb, void *data)
{
    doodle_editor_t *de = SMALLOC(doodle_editor_t);
    RETVIF(de==NULL, NULL);
    service_dump(service);
    de->win = win;
    de->parent = parent;
    de->cb = (cb==NULL ? memo_com_dummy_cb : cb); /* make sure cb is not null, no need to check legitimacy of cb when call */
    de->data = data;

    /* init */
    const char *s = NULL;
    service_get_extra_data(service, "index", &s);
    if (s == NULL) {
        de->record = memo_create_data();
        de->record->id = -1;
        de->record->modi_time = time((time_t *) 0);
    } else {        /* load item */
        de->record = memo_get_data(atoi(s));
	SFREE(s);
    }
    de->stroke_size = 8;
    de->stroke_color = 0xff000000;

    _create_doodle_editor_layout(de, service);
    return (void *)de;
}

void memo_destroy_doodle_editor(void *h_de)
{
    doodle_editor_t *de = (doodle_editor_t *)h_de;
    if (de->h_ss!= NULL) { /* destroy stroke  size selector */
        _stroke_size_selector_del(de);
    }
    if (de->h_cs != NULL) { /* destroy stroke color selector */
        _stroke_color_selector_del(de);
    }
    memo_free_data(de->record);
    evas_object_del(de->body_main);
    SFREE(de);
}
